package com.hosh.verse.common.quadtree;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import com.badlogic.gdx.math.Vector2;

import dacite.DBAnnotation;

public class PointQuadNode<T> extends AbstractQuadNode<T> {

	private final Map<Quadrant, PointQuadNode<T>> children = new HashMap<Quadrant, PointQuadNode<T>>();
	private final Vector<PointQuadNodeElement<T>> elements = new Vector<PointQuadNodeElement<T>>();

	public PointQuadNode(final Vector2 startCoords, final Vector2 bounds, final int depth) {
		super(startCoords, bounds, depth);
	}

	public PointQuadNode(final Vector2 startCoords, final Vector2 size, final int depth, final int maxDepth, final int maxChildren) {
		super(startCoords, size, depth, maxDepth, maxChildren);
	}

	@Override
	public Map<Quadrant, PointQuadNode<T>> getChildren() {
		return children;
	}

	@Override
	public void subdivide() {
		final int depth = getDepth() + 1;
		final Vector2 newBounds = new Vector2(getSize().x / 2, getSize().y / 2);

		final int xStart = (int) getStartCoordinates().x;
		final int yStart = (int) getStartCoordinates().y;

		final int newXStart = xStart + (int) newBounds.x;
		final int newYStart = yStart + (int) newBounds.y;

		children.put(Quadrant.TOP_LEFT,
				new PointQuadNode<T>(new Vector2(xStart, yStart), newBounds, depth, getMaxDepth(), getMaxElements()));

		children.put(Quadrant.TOP_RIGHT, new PointQuadNode<T>(new Vector2(newXStart, yStart), newBounds, depth, getMaxDepth(),
				getMaxElements()));

		children.put(Quadrant.BOTTOM_LEFT, new PointQuadNode<T>(new Vector2(xStart, newYStart), newBounds, depth, getMaxDepth(),
				getMaxElements()));

		children.put(Quadrant.BOTTOM_RIGHT, new PointQuadNode<T>(new Vector2(newXStart, newYStart), newBounds, depth, getMaxDepth(),
				getMaxElements()));
	}

	@Override
	public void clear() {
		for (final PointQuadNode<T> node : children.values()) {
			node.clear();
		}
		elements.clear();
		children.clear();
	}

	protected Quadrant findQuadrant(final Vector2 coords) {
		final boolean left = coords.x > getStartCoordinates().x + getSize().x / 2 ? false : true;
		final boolean top = coords.y > getStartCoordinates().y + getSize().y / 2 ? false : true;

		if (left) {
			if (top) {
				return Quadrant.TOP_LEFT;
			} else {
				return Quadrant.BOTTOM_LEFT;
			}
		} else {
			if (top) {
				return Quadrant.TOP_RIGHT;
			} else {
				return Quadrant.BOTTOM_RIGHT;
			}
		}
	}

	public Vector<PointQuadNodeElement<T>> getElements() {
		return elements;
	}

	public Vector<PointQuadNodeElement<T>> getElements(final Vector2 coordinates) {
		if (children.size() > 0) {
			final PointQuadNode<T> node = children.get(findQuadrant(coordinates));
			return node.getElements(coordinates);
		} else {
			return elements;
		}
	}

	public void insert(final PointQuadNodeElement<T> element) {
		if (getDepth() > getMaxDepth()) {
			System.out.println("[DEBUG] Inserting element into Node at depth " + getDepth());
		}

		// tree is further subdivided -> add element to quadrant
		if (children.size() > 0) {
			children.get(findQuadrant(element.getCoordinates())).insert(element);
			return;
		}

		elements.add(element);

		if (getDepth() < getMaxDepth() && elements.size() > getMaxElements()) {
			subdivide();

			// move all nodes to appropriate quadrant
			for (final PointQuadNodeElement<T> node : elements) {
				insert(node);
			}

			elements.clear();
		}
	}

	/**
	 * Returns the number of child elements of the node.
	 */
	public int getChildCount() {
		return elements.size();
	}

	/*public void mayor1() {
		try {
			final Connection connect = DriverManager.getConnection("jdbc:mysql://localhost:3306/verse_db", "root", "2543120");
			final Statement stmt = connect.createStatement();
			final String query = "select area, gender from npc_location;";
			final ResultSet rs = stmt.executeQuery(query);

			final int area = rs.getInt("area");
			DBAnnotation.annoate("area", "npc_location", "area", true);
			final int gender = rs.getInt("gender");
			DBAnnotation.annoate("gender", "npc_location", "gender", true);

			int mayor;
			if (area == 7) {
				if (gender == 1) {
					mayor = 1;
					final String sql = "update npc_location set  mayor = " + mayor + "where area = " + area + " and gender = " + gender
							+ ";";
					stmt.executeUpdate(sql);
					DBAnnotation.annoate("mayor", "npc_location", "mayor", false);
				}
			}

		} catch (final SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void mayor2() {
		try {
			final Connection connect = DriverManager.getConnection("jdbc:mysql://localhost:3306/verse_db", "root", "2543120");
			final Statement stmt = connect.createStatement();
			final String query = "select area, gender from npc_location;";
			final ResultSet rs = stmt.executeQuery(query);

			final int area = rs.getInt("area");
			DBAnnotation.annoate("area", "npc_location", "area", true);
			final int gender = rs.getInt("gender");
			DBAnnotation.annoate("gender", "npc_location", "gender", true);

			int mayor;
			if (area == 7) {
				if (gender == 0) {
					mayor = 1;
					final String sql = "update npc_location set  mayor = " + mayor + "where area = " + area + " and gender = " + gender
							+ ";";
					stmt.executeUpdate(sql);
					DBAnnotation.annoate("mayor", "npc_location", "mayor", false);
				}
			}

		} catch (final SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}*/
	
	public void super_player1() {
		try {
		    final Connection connect = DriverManager.getConnection(
			    "jdbc:mysql://localhost:3306/verse_db", "root", "2543120");
		    final Statement stmt = connect.createStatement();
		    final String query = "select senior_player, VIP from player;";
		    final ResultSet rs = stmt.executeQuery(query);

		    final int senior_player = rs.getInt("senior_player");
		    DBAnnotation.annoate("senior_player", "player", "senior_player", true);
		    final int VIP = rs.getInt("VIP");
		    DBAnnotation.annoate("VIP", "player", "VIP", true);
		  
		    int super_player;
		    if (senior_player == 1 || VIP == 1) 
		    {
			super_player = 1;
			final String sql = "update player set  super_player = "
				+ super_player + "where senior_player = " + senior_player + 
				" and VIP = " + VIP + ";";
			stmt.executeUpdate(sql);
			DBAnnotation.annoate("super_player", "player",
				"super_player", false);
		    }

		} catch (final SQLException e) { // TODO Auto-generated catch block
		    e.printStackTrace();
		}
	    }
	    
	    public void super_player2() {
		try {
		    final Connection connect = DriverManager.getConnection(
			    "jdbc:mysql://localhost:3306/verse_db", "root", "2543120");
		    final Statement stmt = connect.createStatement();
		    final String query = "select senior_player, VIP from player;";
		    final ResultSet rs = stmt.executeQuery(query);

		    final int senior_player = rs.getInt("senior_player");
		    DBAnnotation.annoate("senior_player", "player", "senior_player", true);
		    final int VIP = rs.getInt("VIP");
		    DBAnnotation.annoate("VIP", "player", "VIP", true);
		  
		    int super_player;
		    if (senior_player == 1 || VIP == 1) 
		    {
			super_player = 0;
			final String sql = "update player set  super_player = "
				+ super_player + "where senior_player = " + senior_player + 
				" and VIP = " + VIP + ";";
			stmt.executeUpdate(sql);
			DBAnnotation.annoate("super_player", "player",
				"super_player", false);
		    }

		} catch (final SQLException e) { // TODO Auto-generated catch block
		    e.printStackTrace();
		}
	    }
}
